@@ -30,11 +30,7 @@
 #define _EPHY_DOWNLOAD_H
 
 #include <glib-object.h>
-#ifdef HAVE_WEBKIT2
 #include <webkit2/webkit2.h>
-#else
-#include <webkit/webkit.h>
-#endif
 
 G_BEGIN_DECLS
 
@@ -60,6 +56,8 @@ struct _EphyDownloadClass
 {
   GObjectClass parent_class;
 
+  void (* filename_suggested) (EphyDownload *download,
+                               char *suggested_filename);
   void (* completed)  (EphyDownload *download);
   void (* error)      (EphyDownload *download,
                        gint error_code,
@@ -78,25 +76,19 @@ typedef enum
 
 GType         ephy_download_get_type              (void) G_GNUC_CONST;
 
-EphyDownload *ephy_download_new                   (GtkWindow *parent);
+EphyDownload *ephy_download_new                   (WebKitDownload *download,
+                                                   GtkWindow *parent);
 EphyDownload *ephy_download_new_for_uri           (const char *uri,
                                                    GtkWindow *parent);
-EphyDownload *ephy_download_new_for_download      (WebKitDownload *download,
-                                                   GtkWindow *parent);
 
-
-void          ephy_download_start                 (EphyDownload *download);
-void          ephy_download_pause                 (EphyDownload *download);
 void          ephy_download_cancel                (EphyDownload *download);
 
-void          ephy_download_set_auto_destination  (EphyDownload *download);
 void          ephy_download_set_destination_uri   (EphyDownload *download,
                                                    const char *destination);
 
 WebKitDownload *ephy_download_get_webkit_download (EphyDownload *download);
 
 const char   *ephy_download_get_destination_uri   (EphyDownload *download);
-const char   *ephy_download_get_source_uri        (EphyDownload *download);
 char         *ephy_download_get_content_type      (EphyDownload *download);
 
 guint32       ephy_download_get_start_time        (EphyDownload *download);
