@@ -54,6 +54,7 @@ struct _EphyEmbedShellPrivate
   GtkPageSetup *page_setup;
   GtkPrintSettings *print_settings;
   EphyEmbedShellMode mode;
+  WebKitUserContentManager *user_content;
   EphyAboutHandler *about_handler;
   guint update_overview_timeout_id;
   guint hiding_overview_item;
@@ -62,6 +63,7 @@ struct _EphyEmbedShellPrivate
   guint web_extensions_page_created_signal_id;
   guint web_extensions_form_auth_save_signal_id;
   guint web_extensions_remove_from_overview_signal_id;
+  guint web_extensions_allow_tls_certificate_signal_id;
 };
 
 enum
@@ -104,6 +106,7 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->print_settings);
   g_clear_object (&priv->global_history_service);
   g_clear_object (&priv->about_handler);
+  g_clear_object (&priv->user_content);
 
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
 }
@@ -123,6 +126,9 @@ ephy_embed_shell_find_web_extension (EphyEmbedShell *shell,
 
   l = g_list_find_custom (shell->priv->web_extensions, name_owner, (GCompareFunc)web_extension_compare);
 
+  if (!l)
+    g_warning ("Could not find extension with name owner `%s´.", name_owner);
+
   return l ? EPHY_WEB_EXTENSION_PROXY (l->data) : NULL;
 }
 
@@ -266,6 +272,26 @@ web_extension_remove_from_overview (GDBusConnection *connection,
     g_timeout_add (OVERVIEW_RELOAD_DELAY, (GSourceFunc) ephy_embed_shell_update_overview_timeout_cb, shell);
 }
 
+static void
+web_extension_allow_tls_certificate (GDBusConnection *connection,
+                                     const char *sender_name,
+                                     const char *object_path,
+                                     const char *interface_name,
+                                     const char *signal_name,
+                                     GVariant *parameters,
+                                     EphyEmbedShell *shell)
+{
+  EphyWebExtensionProxy *web_extension;
+  guint64 page_id;
+
+  web_extension = ephy_embed_shell_find_web_extension (shell, sender_name);
+  if (!web_extension)
+    return;
+
+  g_variant_get (parameters, "(t)", &page_id);
+  ephy_web_extension_proxy_allow_tls_certificate (web_extension, page_id);
+}
+
 static void
 web_extension_destroyed (EphyEmbedShell *shell,
                          GObject *web_extension)
@@ -540,6 +566,17 @@ ephy_embed_shell_setup_web_extensions_connection (EphyEmbedShell *shell)
                                         (GDBusSignalCallback)web_extension_remove_from_overview,
                                         shell,
                                         NULL);
+  shell->priv->web_extensions_allow_tls_certificate_signal_id =
+    g_dbus_connection_signal_subscribe (shell->priv->bus,
+                                        NULL,
+                                        EPHY_WEB_EXTENSION_INTERFACE,
+                                        "AllowTLSCertificate",
+                                        EPHY_WEB_EXTENSION_OBJECT_PATH,
+                                        NULL,
+                                        G_DBUS_SIGNAL_FLAGS_NONE,
+                                        (GDBusSignalCallback)web_extension_allow_tls_certificate,
+                                        shell,
+                                        NULL);
 }
 
 static void
@@ -641,9 +678,14 @@ ephy_embed_shell_shutdown (GApplication* application)
     priv->web_extensions_remove_from_overview_signal_id = 0;
   }
 
+  if (priv->web_extensions_allow_tls_certificate_signal_id > 0) {
+    g_dbus_connection_signal_unsubscribe (priv->bus, priv->web_extensions_allow_tls_certificate_signal_id);
+    priv->web_extensions_allow_tls_certificate_signal_id = 0;
+  }
+
   g_list_foreach (priv->web_extensions, (GFunc)ephy_embed_shell_unwatch_web_extension, application);
 
-  g_object_unref (ephy_embed_prefs_get_web_view_group ());
+  g_object_unref (ephy_embed_prefs_get_settings ());
   ephy_embed_utils_shutdown ();
 }
 
@@ -861,7 +903,7 @@ ephy_embed_shell_get_page_setup (EphyEmbedShell *shell)
 }
 
 /**
- * ephy_embed_shell_set_print_gettings:
+ * ephy_embed_shell_set_print_settings:
  * @shell: the #EphyEmbedShell
  * @settings: the new #GtkPrintSettings object
  *
@@ -941,7 +983,7 @@ ephy_embed_shell_get_mode (EphyEmbedShell *shell)
 }
 
 /**
- * ephy_embed_shell_launch_application:
+ * ephy_embed_shell_launch_handler:
  * @shell: an #EphyEmbedShell
  * @file: a #GFile to open
  * @mime_type: the mime type of @file or %NULL
@@ -995,3 +1037,12 @@ ephy_embed_shell_clear_cache (EphyEmbedShell *shell)
 {
   webkit_web_context_clear_cache (webkit_web_context_get_default ());
 }
+
+WebKitUserContentManager *
+ephy_embed_shell_get_user_content_manager (EphyEmbedShell *shell)
+{
+  if (!shell->priv->user_content)
+    shell->priv->user_content = webkit_user_content_manager_new ();
+
+  return shell->priv->user_content;
+}
