@@ -526,11 +526,10 @@ ephy_web_view_create_form_auth_save_confirmation_info_bar (EphyWebView *web_view
 }
 
 static void
-update_navigation_flags (EphyWebView *view)
+update_navigation_flags (WebKitWebView *web_view)
 {
-  EphyWebViewPrivate *priv = view->priv;
+  EphyWebViewPrivate *priv = EPHY_WEB_VIEW (web_view)->priv;
   guint flags = 0;
-  WebKitWebView *web_view = WEBKIT_WEB_VIEW (view);
 
   if (webkit_web_view_can_go_back (web_view))
     flags |= EPHY_WEB_VIEW_NAV_BACK;
@@ -541,7 +540,7 @@ update_navigation_flags (EphyWebView *view)
   if (priv->nav_flags != (EphyWebViewNavigationFlags)flags) {
     priv->nav_flags = (EphyWebViewNavigationFlags)flags;
 
-    g_object_notify (G_OBJECT (view), "navigation");
+    g_object_notify (G_OBJECT (web_view), "navigation");
   }
 }
 
@@ -732,13 +731,13 @@ page_created_cb (EphyEmbedShell *shell,
   priv->web_extension = web_extension;
   g_object_add_weak_pointer (G_OBJECT (priv->web_extension), (gpointer *)&priv->web_extension);
 
-  g_signal_connect (priv->web_extension, "form-auth-data-save-requested",
-                    G_CALLBACK (form_auth_data_save_requested),
-                    web_view);
+  g_signal_connect_object (priv->web_extension, "form-auth-data-save-requested",
+                           G_CALLBACK (form_auth_data_save_requested),
+                           web_view, 0);
 
-  g_signal_connect (priv->web_extension, "allow-tls-certificate",
-                    G_CALLBACK (allow_tls_certificate_cb),
-                    web_view);
+  g_signal_connect_object (priv->web_extension, "allow-tls-certificate",
+                           G_CALLBACK (allow_tls_certificate_cb),
+                           web_view, 0);
 }
 
 static void
@@ -750,10 +749,6 @@ ephy_web_view_dispose (GObject *object)
                                         ephy_web_view_history_cleared_cb,
                                         EPHY_WEB_VIEW (object));
 
-  g_signal_handlers_disconnect_by_func (object, icon_changed_cb, NULL);
-
-  g_signal_handlers_disconnect_by_func (ephy_embed_shell_get_default (), form_auth_data_save_requested, object);
-
   g_clear_object (&priv->file_monitor);
 
   g_clear_object (&priv->icon);
@@ -921,6 +916,8 @@ ephy_web_view_constructed (GObject *object)
 
   g_signal_connect (object, "web-process-crashed",
                     G_CALLBACK (process_crashed_cb), object);
+  g_signal_connect_swapped (webkit_web_view_get_back_forward_list (WEBKIT_WEB_VIEW (object)),
+                            "changed", G_CALLBACK (update_navigation_flags), object);
 }
 
 static void
@@ -1553,8 +1550,6 @@ ephy_web_view_location_changed (EphyWebView *view,
 
   _ephy_web_view_update_icon (view);
 
-  update_navigation_flags (view);
-
   g_object_thaw_notify (object);
 }
 
@@ -1962,7 +1957,6 @@ load_failed_cb (WebKitWebView *web_view,
 
   priv->load_failed = TRUE;
   ephy_web_view_set_link_message (view, NULL);
-  update_navigation_flags (view);
 
   if (error->domain == SOUP_HTTP_ERROR) {
     ephy_web_view_load_error_page (view, uri, EPHY_WEB_VIEW_ERROR_PAGE_NETWORK_ERROR, error);
@@ -2108,9 +2102,9 @@ ephy_web_view_init (EphyWebView *web_view)
   priv->history_service = EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ()));
   priv->history_service_cancellable = g_cancellable_new ();
 
-  g_signal_connect (priv->history_service,
-                    "cleared", G_CALLBACK (ephy_web_view_history_cleared_cb),
-                    web_view);
+  g_signal_connect_object (priv->history_service,
+                           "cleared", G_CALLBACK (ephy_web_view_history_cleared_cb),
+                           web_view, 0);
 
   g_signal_connect (web_view, "decide-policy",
                     G_CALLBACK (decide_policy_cb),
