@@ -62,9 +62,9 @@ struct _EphyShellPrivate {
   GtkWidget *bme;
   GtkWidget *history_window;
   GObject *prefs_dialog;
-  GList *del_on_exit;
-  EphyShellStartupContext *startup_context;
-  guint open_uris_idle_id;
+  EphyShellStartupContext *local_startup_context;
+  EphyShellStartupContext *remote_startup_context;
+  GSList *open_uris_idle_ids;
 };
 
 EphyShell *ephy_shell = NULL;
@@ -113,16 +113,27 @@ ephy_shell_startup_context_new (EphyStartupFlags startup_flags,
 }
 
 static void
-ephy_shell_startup_continue (EphyShell *shell)
+ephy_shell_startup_context_free (EphyShellStartupContext *ctx)
+{
+  g_assert (ctx != NULL);
+
+  g_free (ctx->bookmarks_filename);
+  g_free (ctx->session_filename);
+  g_free (ctx->bookmark_url);
+
+  g_strfreev (ctx->arguments);
+
+  g_slice_free (EphyShellStartupContext, ctx);
+}
+
+static void
+ephy_shell_startup_continue (EphyShell *shell, EphyShellStartupContext *ctx)
 {
-  EphyShellStartupContext *ctx;
   EphySession *session;
 
   session = ephy_shell_get_session (shell);
   g_assert (session != NULL);
 
-  ctx = shell->priv->startup_context;
-
   if (ctx->session_filename != NULL)
     ephy_session_load (session, (const char *)ctx->session_filename,
                        ctx->user_time, NULL, NULL, NULL);
@@ -230,7 +241,7 @@ quit_application (GSimpleAction *action,
 }
 
 static GActionEntry app_entries[] = {
-  { "new", new_window, NULL, NULL, NULL },
+  { "new-window", new_window, NULL, NULL, NULL },
   { "new-incognito", new_incognito_window, NULL, NULL, NULL },
   { "bookmarks", show_bookmarks, NULL, NULL, NULL },
   { "history", show_history, NULL, NULL, NULL },
@@ -289,29 +300,32 @@ session_load_cb (GObject *object,
                  gpointer user_data)
 {
   EphySession *session = EPHY_SESSION (object);
-  EphyShell *shell = EPHY_SHELL (user_data);
+  EphyShellStartupContext *ctx = (EphyShellStartupContext *)user_data;
 
   ephy_session_resume_finish (session, result, NULL);
-  ephy_shell_startup_continue (shell);
+  ephy_shell_startup_continue (ephy_shell_get_default (), ctx);
 }
 
 static void
 ephy_shell_activate (GApplication *application)
 {
   EphyShell *shell = EPHY_SHELL (application);
+  EphyShellPrivate *priv = shell->priv;
 
   /*
    * We get here on each new instance (remote or not). Autoresume the
-   * session unless we are in application mode and queue the commands.
+   * session and queue the commands if we are a local instance. Otherwise,
+   * execute the commands immediately, before the remote startup context
+   * can be invalidated by another remote instance.
    */
-  if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell)) != EPHY_EMBED_SHELL_MODE_APPLICATION) {
-    EphyShellStartupContext *ctx;
-
-    ctx = shell->priv->startup_context;
+  if (priv->remote_startup_context == NULL) {
     ephy_session_resume (ephy_shell_get_session (shell),
-                         ctx->user_time, NULL, session_load_cb, shell);
-  } else
-    ephy_shell_startup_continue (shell);
+                         priv->local_startup_context->user_time,
+                         NULL, session_load_cb, priv->local_startup_context);
+  } else {
+    ephy_shell_startup_continue (shell, priv->remote_startup_context);
+    g_clear_pointer (&priv->remote_startup_context, ephy_shell_startup_context_free);
+  }
 }
 
 /*
@@ -343,13 +357,13 @@ ephy_shell_add_platform_data (GApplication *application,
   G_APPLICATION_CLASS (ephy_shell_parent_class)->add_platform_data (application,
                                                                     builder);
 
-  if (app->priv->startup_context) {
+  if (app->priv->local_startup_context) {
     /*
      * We create an array variant that contains only the elements in
      * ctx that are non-NULL.
      */
     ctx_builder = g_variant_builder_new (G_VARIANT_TYPE_ARRAY);
-    ctx = app->priv->startup_context;
+    ctx = app->priv->local_startup_context;
 
     if (ctx->startup_flags)
       g_variant_builder_add (ctx_builder, "{iv}",
@@ -396,24 +410,6 @@ ephy_shell_add_platform_data (GApplication *application,
   }
 }
 
-static void
-ephy_shell_free_startup_context (EphyShell *shell)
-{
-  EphyShellStartupContext *ctx = shell->priv->startup_context;
-
-  g_assert (ctx != NULL);
-
-  g_free (ctx->bookmarks_filename);
-  g_free (ctx->session_filename);
-  g_free (ctx->bookmark_url);
-
-  g_strfreev (ctx->arguments);
-
-  g_slice_free (EphyShellStartupContext, ctx);
-
-  shell->priv->startup_context = NULL;
-}
-
 static void
 ephy_shell_before_emit (GApplication *application,
                         GVariant *platform_data)
@@ -464,9 +460,9 @@ ephy_shell_before_emit (GApplication *application,
     }
   }
 
-  if (shell->priv->startup_context)
-    ephy_shell_free_startup_context (shell);
-  shell->priv->startup_context = ctx;
+  /* We have already processed and discarded any previous remote startup contexts. */
+  g_assert (shell->priv->remote_startup_context == NULL);
+  shell->priv->remote_startup_context = ctx;
 
   G_APPLICATION_CLASS (ephy_shell_parent_class)->before_emit (application,
                                                               platform_data);
@@ -597,6 +593,12 @@ ephy_shell_init (EphyShell *shell)
   g_free (favicon_db_path);
 }
 
+static void
+remove_open_uris_idle_cb (gpointer data)
+{
+  g_source_remove (GPOINTER_TO_UINT (data));
+}
+
 static void
 ephy_shell_dispose (GObject *object)
 {
@@ -612,10 +614,8 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&priv->bookmarks);
   g_clear_object (&priv->network_monitor);
 
-  if (priv->open_uris_idle_id > 0)  {
-    g_source_remove (priv->open_uris_idle_id);
-    priv->open_uris_idle_id = 0;
-  }
+  g_slist_free_full (priv->open_uris_idle_ids, remove_open_uris_idle_cb);
+  priv->open_uris_idle_ids = NULL;
 
   G_OBJECT_CLASS (ephy_shell_parent_class)->dispose (object);
 }
@@ -624,9 +624,10 @@ static void
 ephy_shell_finalize (GObject *object)
 {
   EphyShell *shell = EPHY_SHELL (object);
+  EphyShellPrivate *priv = shell->priv;
 
-  if (shell->priv->startup_context)
-    ephy_shell_free_startup_context (shell);
+  g_clear_pointer (&priv->local_startup_context, ephy_shell_startup_context_free);
+  g_clear_pointer (&priv->remote_startup_context, ephy_shell_startup_context_free);
 
   G_OBJECT_CLASS (ephy_shell_parent_class)->finalize (object);
 
@@ -875,7 +876,7 @@ _ephy_shell_create_instance (EphyEmbedShellMode mode)
  * @shell: A #EphyShell
  * @ctx: (transfer full): a #EphyShellStartupContext
  *
- * Sets the startup context to be used during activation of a new instance.
+ * Sets the local startup context to be used during activation of a new instance.
  * See ephy_shell_set_startup_new().
  **/
 void
@@ -884,10 +885,9 @@ ephy_shell_set_startup_context (EphyShell *shell,
 {
   g_return_if_fail (EPHY_IS_SHELL (shell));
 
-  if (shell->priv->startup_context)
-    ephy_shell_free_startup_context (shell);
+  g_assert (shell->priv->local_startup_context == NULL);
 
-  shell->priv->startup_context = ctx;
+  shell->priv->local_startup_context = ctx;
 }
 
 guint
@@ -981,6 +981,7 @@ typedef struct {
   EphyEmbed *previous_embed;
   guint current_uri;
   gboolean reuse_empty_tab;
+  guint source_id;
 } OpenURIsData;
 
 static OpenURIsData *
@@ -1082,7 +1083,8 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
 static void
 ephy_shell_open_uris_idle_done (OpenURIsData *data)
 {
-  data->shell->priv->open_uris_idle_id = 0;
+  data->shell->priv->open_uris_idle_ids = g_slist_remove (data->shell->priv->open_uris_idle_ids,
+                                                          GUINT_TO_POINTER (data->source_id));
   open_uris_data_free (data);
 }
 
@@ -1092,14 +1094,18 @@ ephy_shell_open_uris (EphyShell *shell,
                       EphyStartupFlags startup_flags,
                       guint32 user_time)
 {
+  OpenURIsData *data;
+  guint id;
+
   g_return_if_fail (EPHY_IS_SHELL (shell));
 
-  if (shell->priv->open_uris_idle_id == 0) {
-    shell->priv->open_uris_idle_id =
-      g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
-                       (GSourceFunc)ephy_shell_open_uris_idle,
-                       open_uris_data_new (shell, uris, startup_flags, user_time),
-                       (GDestroyNotify)ephy_shell_open_uris_idle_done);
-  }
+  data = open_uris_data_new (shell, uris, startup_flags, user_time);
+  id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                        (GSourceFunc)ephy_shell_open_uris_idle,
+                        data,
+                        (GDestroyNotify)ephy_shell_open_uris_idle_done);
+  data->source_id = id;
+
+  shell->priv->open_uris_idle_ids = g_slist_prepend (shell->priv->open_uris_idle_ids, GUINT_TO_POINTER (id));
 }
 
