@@ -691,6 +691,7 @@ language_editor_add_button_clicked_cb (GtkWidget *button,
 		GtkDialog **add_lang_dialog;
 
 		pd->priv->add_lang_dialog = setup_add_language_dialog (pd);
+		gtk_window_set_transient_for (GTK_WINDOW (pd->priv->add_lang_dialog), GTK_WINDOW (pd));
 
 		add_lang_dialog = &pd->priv->add_lang_dialog;
 
@@ -1027,7 +1028,7 @@ search_engine_combo_add_default_engines (GtkListStore *store)
 					     /* Search engine option in the preferences dialog */
 					     { N_("Bing"),
 					       /* For the preferences dialog. Consider a regional variant, like uk.bing.com */
-					       N_("http://www.bing.com/search?q=%s")} };
+					       N_("https://www.bing.com/search?q=%s")} };
 
 	for (i = 0; i < G_N_ELEMENTS (default_engines); ++i)
 	{
